import { getContextItem } from "./context.ts";

export abstract class Drawable {
  abstract draw(...args: unknown[]): void;
}

export abstract class Debuggable extends Drawable {
  constructor(...debugKeys: [boolean | keyof Debug, ...(keyof Debug)[]]) {
    super();
    if (import.meta.env.DEV) {
      let drawFirst = false;
      if (typeof debugKeys[0] === "boolean") {
        drawFirst = debugKeys.shift() as boolean;
      }
      const draw = this.draw.bind(this);
      this.draw = drawFirst
        ? (...args: unknown[]) => {
          draw(...args);
          const debug = getContextItem<Debug>("debug");
          if (debugKeys.some((k) => debug[k as keyof Debug])) {
            this.debugDraw();
          }
        }
        : (...args: unknown[]) => {
          const debug = getContextItem<Debug>("debug");
          if (debugKeys.some((k) => debug[k as keyof Debug])) {
            this.debugDraw();
          }
          draw(...args);
        };
    }
  }

  abstract debugDraw(...args: unknown[]): void;
}
