import { Vector } from "@bearmetal/doodler";

declare global {
  type GridItem = [Vector, Vector, Set<tag>];
}
export class SpatialHashGrid {
  private grid: Map<string, GridItem[]> = new Map();
  private cellSize: number;

  constructor(cellSize: number) {
    this.cellSize = cellSize;
  }

  private getKey(x: number, y: number): string {
    return `${Math.floor(x / this.cellSize)},${Math.floor(y / this.cellSize)}`;
  }

  insert(segment: GridItem) {
    const [a, b] = segment;
    const minX = Math.min(a.x, b.x);
    const minY = Math.min(a.y, b.y);
    const maxX = Math.max(a.x, b.x);
    const maxY = Math.max(a.y, b.y);

    for (let x = minX; x <= maxX; x += this.cellSize) {
      for (let y = minY; y <= maxY; y += this.cellSize) {
        const key = this.getKey(x, y);
        if (!this.grid.has(key)) this.grid.set(key, []);
        this.grid.get(key)!.push(segment);
      }
    }
  }

  query(position: Vector, radius: number, tag?: tag): GridItem[] {
    const minX = position.x - radius;
    const minY = position.y - radius;
    const maxX = position.x + radius;
    const maxY = position.y + radius;

    const segments: Set<GridItem> = new Set();

    for (let x = minX; x <= maxX; x += this.cellSize) {
      for (let y = minY; y <= maxY; y += this.cellSize) {
        const key = this.getKey(x, y);
        if (this.grid.has(key)) {
          for (const segment of this.grid.get(key)!) {
            tag
              ? segment[2].has(tag) &&
                segments.add(segment)
              : segments.add(segment);
          }
        }
      }
    }

    return Array.from(segments);
  }

  getAllSegments() {
    return Array.from(this.grid.values()).flatMap((s) => s);
  }

  clear() {
    this.grid.clear();
  }
}
