import { Doodler, Point, Vector, ZoomableDoodler } from "@bearmetal/doodler";
import { getContext, getContextItem } from "../../lib/context.ts";
import { InputManager } from "../../lib/input.ts";
import { TrackSystem } from "../../track/system.ts";
import { Tender } from "../../train/cars.ts";
import { RedEngine } from "../../train/engines.ts";
// import { Train } from "../../train/train.ts";
import { Train } from "../../train/newTrain/Train.ts";
import { Bogie, Driver } from "../../train/newTrain/Bogie.ts";
import { TrainCar } from "../../train/newTrain/TrainCar.ts";
import { State } from "../machine.ts";
import { States } from "./index.ts";
import { LargeLady, LargeLadyTender } from "../../train/LargeLady.ts";

export class RunningState extends State<States> {
  override name: States = States.RUNNING;
  override validTransitions: Set<States> = new Set([
    States.PAUSED,
    States.EDIT_TRACK,
  ]);

  layers: number[] = [];

  activeTrain: Train | undefined;

  override update(dt: number): void {
    const ctx = getContext() as { trains: Train[]; track: TrackSystem };
    const doodler = getContextItem<ZoomableDoodler>(
      "doodler",
    );
    // if (this.activeTrain) {
    //   // (doodler as any).origin = doodler.worldToScreen(
    //   //   doodler.width - this.activeTrain.aabb.center.x,
    //   //   doodler.height - this.activeTrain.aabb.center.y,
    //   // );
    //   doodler.centerCameraOn(this.activeTrain.aabb.center);
    // }
    // const ctx = getContext() as { trains: DotFollower[]; track: TrackSystem };
    // TODO
    // Update trains
    // Update world
    // Handle input
    // Monitor world events
    for (const train of ctx.trains) {
      train.update(dt);
    }
  }
  override start(): void {
    console.log("Starting running state");
    const doodler = getContextItem<Doodler>("doodler");
    this.layers.push(
      doodler.createLayer(() => {
        const track = getContextItem<TrackSystem>("track");
        track.draw();
      }),
      doodler.createLayer(() => {
        const trains = getContextItem<Train[]>("trains");
        for (const train of trains) {
          // if (input.getKeyState("ArrowUp")) {
          //   train.acceleration.x += 10;
          // }
          train.draw();
        }
      }),
    );

    // noop
    const inputManager = getContextItem<InputManager>("inputManager");
    const track = getContextItem<TrackSystem>("track");
    const ctx = getContext() as { trains: Train[] };
    // const ctx = getContext() as { trains: DotFollower[] };
    inputManager.onKey(" ", () => {
      // const path = track.path;
      // const follower = new DotFollower(path, path.points[0].copy());
      // ctx.trains.push(follower);
      // const train = new Train(track.path, [
      //   new LargeLady(),
      //   new LargeLadyTender(),
      // ]);
      const bogies = [new Bogie(20, 20, 1), new Driver(20, 20, 1)];
      const train = new Train(track, [
        new TrainCar(
          bogies,
        ),
      ]);
      track.generatePath();
      const firstPoint = track.path.points[0].copy();
      let prevOffset = 0;
      for (let i = 0; i < bogies.length; i++) {
        const b = bogies[i];
        b.position = firstPoint.add(b.leadingOffset + prevOffset, 0);
        b.prevPos = b.position.copy();
        prevOffset += b.trailingOffset;
      }
      ctx.trains.push(train);
    });

    inputManager.onKey("ArrowUp", () => {
      const trains = getContextItem<Train[]>("trains");
      for (const train of trains) {
        train.bogies.filter((b) => b instanceof Driver).forEach((b) => {
          b.drivingForce += 10;
          if (b.dir.mag() < 0.1) b.updateDirection(new Vector(1, 0));
          b.velocity = new Vector(10, 0);
        });
      }
    });
    inputManager.onKey("ArrowDown", () => {
      const trains = getContextItem<Train[]>("trains");
      for (const train of trains) {
        train.bogies.filter((b) => b instanceof Driver).forEach((b) => {
          b.drivingForce -= 10;
        });
      }
    });
  }
  override stop(): void {
    for (const layer of this.layers) {
      getContextItem<Doodler>("doodler").deleteLayer(layer);
    }
  }
}
