import { Vector } from "@bearmetal/doodler";
import { getContextItem } from "../../lib/context.ts";
import { Debuggable } from "../../lib/debuggable.ts";
import { Bogie } from "./Bogie.ts";

export class TrainCar extends Debuggable {
  private spriteImg: HTMLImageElement;
  private sprite?: ISprite;
  constructor(
    public bogies: Bogie[] = [],
  ) {
    super("car");
    const res = getContextItem("resources");
    this.spriteImg = res.get("snr:sprite/engine");
    this.sprite = {
      at: new Vector(80, 20),
      width: 70,
      height: 20,
    };
  }

  draw(): void {
    const doodler = getContextItem("doodler");
    for (const b of this.bogies) {
      doodler.drawCircle(b.position, 4, { color: "blue" });
      doodler.fillText(
        b.velocity.mag().toFixed(1).toString(),
        b.position.copy().add(10, 10),
        100,
        {
          color: "white",
        },
      );
    }
    const angle = Vector.sub(this.bogies[1].position, this.bogies[0].position)
      .heading();
    const origin = Vector.lerp(
      this.bogies[0].position,
      this.bogies[1].position,
      .5,
    );
    doodler.drawRotated(origin, angle, () => {
      this.sprite
        ? doodler.drawSprite(
          this.spriteImg,
          this.sprite.at,
          this.sprite.width,
          this.sprite.height,
          origin.copy().sub(
            this.sprite.width / 2,
            this.sprite.height / 2,
          ),
          this.sprite.width,
          this.sprite.height,
        )
        : doodler.drawImage(
          this.spriteImg,
          origin.copy().sub(
            this.spriteImg.width / 2,
            this.spriteImg.height / 2,
          ),
        );
    });
  }

  debugDraw(): void {
  }
}
